      SUBROUTINE ALN1R2(A,B,COS,SIN,SIG)
C
C*********************************************************
C
C    PURPOSE:
C    CALCULATION OF ORTHOGONAL ROTATON MATRIX OF GIVENS.
C
C    METHOD:
C       FOR GIVEN 2-VECTOR  ( A ) WE
C                           ( B )
C
C       COMPUTE MATRIX  G = (  COS , SIN  ) SO THAT
C                           ( -SIN , COS  )
C
C     (  COS     SIN ) ( A ) = ( SQRT(A**2+B**2) )
C     ( -SIN     COS ) ( B ) = ( 0               ).
C
C   COMPUTE SIG = SQRT(A**2+B**2).
C   SIG IS COMPUTED LAST TO ALLOW FOR THE POSSIBILITY
C   THAT SIG MAY BE IN THE SAME LOCATION AS A OR B.
C   SUBROUTINE ALN1R2 IS MODIFICATION OF G1 FROM [1].
C
C    REFERENCES:
C    1.LAWSON C.L. AND HANSON R.J. SOLVING LEAST SQUARES
C    PROBLEMS.- PRENTICE-HALL,INC.,ENGLEWOOD CLIFFS,NEW
C    JERSEY,1974.
C
C   USAGE:
C       CALL ALN1R2(A,B,COS,SIN,SIG)
C
C     INPUT PARAMETERS:
C  A - REAL VARIABLE- THE FIRST COMPONENT OF VECTOR;
C  B - REAL VARIABLE - THE SECOND COMPONENT OF VECTOR
C
C     OUTPUT PARAMETERS:
C  COS -REAL VARIABLE-THE FIRST ELEMENT OF GIVENS MATRIX;
C  SIN -REAL VARIABLE-THE SECOND ELEMENT OF GIVENS MATRIX
C  SIG -REAL VARIABLE-THE EVALUATED NORM OF 2-VECTOR
C       (A,B).
C
C     STANDARD FUNCTIONS AND PROCEDURES: ABS , SQRT .
C
C*********************************************************
C
      REAL A,B,COS,SIG,SIN
      REAL ONE,XR,YR,ZERO
      DATA ONE/1.0E0/,ZERO/0.0E0/
C
C      SEARCH OF A MAXIMAL COMPONENT OF ( A , B ).
C
      IF(ABS(A) .LE. ABS(B)) GO TO 10
C
C    A  IS A MAXIMAL COMPONENT.
C    NORMALIZATION TO PREVENT THE OVERFLOW OR UNDERFLOW.
C
      XR=B/A
      YR=SQRT(ONE+XR**2)
      COS=SIGN(ONE/YR,A)
      SIN=COS*XR
      SIG=ABS(A)*YR
      GO TO 40
C
C       CHECK OF NONZERO VALUE OF  B .
C
   10 IF(B) 20,30,20
C
C    B  IS THE MAXIMAL COMPONENT.
C    NORMALIZATION FOR PREVENT THE OVERFLOW OR UNDERFLOW.
C
   20 XR=A/B
      YR=SQRT(ONE+XR**2)
      SIN=SIGN(ONE/YR,B)
      COS=SIN*XR
      SIG=ABS(B)*YR
      GO TO 40
C
C        BOTH  A  AND  B  ARE ZERO'S.
C
   30 SIG=ZERO
      COS=ZERO
      SIN=ONE
C
C       END OF CALCULATION OF GIVENS ROTATION MATRIX.
C
   40 RETURN
      END
